<?php
/* 
 * Phake - Mocking Framework
 * 
 * Copyright (c) 2010-2012, Mike Lively <m@digitalsandwich.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 *  *  Neither the name of Mike Lively nor the names of his
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * @category   Testing
 * @package    Phake
 * @author     Mike Lively <m@digitalsandwich.com>
 * @copyright  2010 Mike Lively <m@digitalsandwich.com>
 * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @link       http://www.digitalsandwich.com/
 */

class PHake_Matchers_MethodMatcherTest extends PHPUnit_Framework_TestCase
{
    /**
     * @var Phake_Matchers_MethodMatcher
     */
    private $matcher;

    /**
     * @var array
     */
    private $arguments;

    public function setUp()
    {
        $this->arguments = array(
            $this->getMock('Phake_Matchers_IArgumentMatcher'),
            $this->getMock('Phake_Matchers_IArgumentMatcher')
        );
        $this->matcher   = new Phake_Matchers_MethodMatcher('foo', $this->arguments);
    }

    /**
     * Tests that passing invalid arguments to the Method Matcher will cause an exception
     *
     * @expectedException InvalidArgumentException
     */
    public function testInvalidParametersThrow()
    {
        new Phake_Matchers_MethodMatcher('foo', array('blah'));
    }

    /**
     * Tests that the method matcher will forward arguments on.
     */
    public function testMatchesForwardsParameters()
    {
        $arguments = array($this->getMock('Phake_Matchers_IArgumentMatcher'));
        $matcher   = new Phake_Matchers_MethodMatcher('foo', $arguments);
        $arguments[0]->expects($this->any())
            ->method('matches')
            ->with($this->equalTo('foo'))
            ->will($this->returnValue(true));

        $arguments = array('foo');
        $matcher->matches('foo', $arguments);
    }

    /**
     * Tests that the method matcher will return true when all is well.
     */
    public function testMatchesSuccessfullyMatches()
    {
        $this->arguments[0]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(true));

        $this->arguments[1]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(true));

        $arguments = array('foo', 'bar');
        $this->assertTrue($this->matcher->matches('foo', $arguments));
    }

    /**
     * Tests that the matcher will return false on mismatched method name.
     */
    public function testNoMatcherOnBadMethod()
    {
        $this->arguments[0]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(true));

        $this->arguments[1]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(true));

        $arguments = array('foo', 'bar');
        $this->assertFalse($this->matcher->matches('test', $arguments));
    }

    /**
     * Tests that the matcher will return false on mismatched argument 1.
     */
    public function testNoMatcherOnBadArg1()
    {
        $this->arguments[0]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(false));

        $this->arguments[1]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(true));

        $arguments = array('foo', 'bar');
        $this->assertFalse($this->matcher->matches('foo', $arguments));
    }

    /**
     * Tests that the matcher will return false on mismatched argument 2.
     */
    public function testNoMatcherOnBadArg2()
    {
        $this->arguments[0]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(true));

        $this->arguments[1]->expects($this->any())
            ->method('matches')
            ->will($this->returnValue(false));

        $arguments = array('foo', 'bar');
        $this->assertFalse($this->matcher->matches('foo', $arguments));
    }

    public function testAnyParameterMatching()
    {
        $matcher = new Phake_Matchers_MethodMatcher('method', array(new Phake_Matchers_AnyParameters()));

        $arguments = array(1, 2, 3);
        $this->assertTrue($matcher->matches('method', $arguments));
        $arguments = array(2, 3, 4);
        $this->assertTrue($matcher->matches('method', $arguments));
        $arguments = array(3, 4, 5);
        $this->assertTrue($matcher->matches('method', $arguments));
    }

    public function testSetterMatcher()
    {
        $matcher = new Phake_Matchers_MethodMatcher('method', array(new Phake_Matchers_ReferenceSetter(42)));

        $value        = 'blah';
        $arguments    = array();
        $arguments[0] =& $value;

        $matcher->matches('method', $arguments);

        $this->assertEquals(42, $value);
    }
}


